#Author: Michal Pavelka, pavelka@karlin.mff.cuni.cz
#Run: python SO3.py

import numpy as np
import math
from RigidBody import *
#import random

N = 100000 #number of steps

#initial condition
mx = 0.1
my = 1.0
mz = 0.1
print "Initial |m| = ", math.sqrt(mx**2+my**2+mz**2)

#Moments of inertia
Ix = 1.0
Iy = 5.0
Iz = 10.0

#time steps
dt= 0.01 
tau = dt * 100.0
print "dt = ", dt
print "tau = ", tau

#Load solver
RB = RBSeReFE(Ix, Iy, Iz, mx, my, mz, dt, tau)
#RB= RBESeReCN(Ix, Iy, Iz, mx, my, mz, dt, tau)

print "Initial Omega_x = ", RB.omega_x()
print "Initial Omega_y = ", RB.omega_y()
print "Initial Omega_z = ", RB.omega_z()

print "Initial internal entropy =", RB.sin

print "Initial internal energy = ", RB.Ein()
print "Initial E_kin_x = ", RB.energy_x()
print "Initial E_kin_y = ", RB.energy_y()
print "Initial E_kin_z = ", RB.energy_z()

print "Initial kinetic E = ", RB.energy()
print "Initial Phi_x = ", RB.Phi_x()
print "Initial Phi_z = ", RB.Phi_z()


#preparing files for export
mfile = open("m.xyz",'w')
def store_m(values):
    result = str(values[0])
    for i in range(1, len(values)):
        result += " "+str(values[i])
    result += "\n"
    mfile.write(result)

store_m([0, mx,my,mz])
store_each = 1


try:
    #calculate evolution
    for i in range(N-1):
        #SeRe:
        #mx_new = mx + dt*my*mz*Jx + 0.5*dt*dtau*mx*(my*my * Jx*Jz + mz*mz * Jx*Jy)
        #my_new = my + dt*mz*mx*Jy + 0.5*dt*dtau*my*(mz*mz * Jy*Jx + mx*mx * Jy*Jz)
        #mz_new = mz + dt*mx*my*Jz + 0.5*dt*dtau*mz*(mx*mx * Jz*Jy + my*my * Jz*Jx)

        #Energetic SeRe explicit:
        #mx_new = mx + dt*my*mz*Jx + 0.5*dt*dtau*mx*(my*my * Jz/Iz - mz*mz*Jy/Iy)
        #my_new = my + dt*mz*mx*Jy + 0.5*dt*dtau*my*(mz*mz * Jx/Ix - mx*mx*Jz/Iz)
        #mz_new = mz + dt*mx*my*Jz + 0.5*dt*dtau*mz*(mx*mx * Jy/Iy - my*my*Jx/Ix)

        #Entropic SeRe explicit:
        #mx_new = mx + dt*my*mz*Jx + 0.5*dt*dtau*mx*(-my*my * Jz/Iy + mz*mz*Jy/Iz)
        #my_new = my + dt*mz*mx*Jy + 0.5*dt*dtau*my*(-mz*mz * Jx/Iz + mx*mx*Jz/Ix)
        #mz_new = mz + dt*mx*my*Jz + 0.5*dt*dtau*mz*(-mx*mx * Jy/Ix + my*my*Jx/Iy)

        #Energetic SeRe Crank-Nicolson:
        #...
        
        #Find new m
        RB.m_new(with_entropy = True)


        #si_new = si+ 0.5*dtau*dt/Ein_s(si) * ((my*mz*Jx)**2/Ix + (mz*mx*Jy)**2/Iy + (mx*my*Jz)**2/Iz)

        #mx = mx_new
        #my = my_new
        #mz = mz_new
        #si = si_new

        if i % store_each == 0: #store the value in file
            t = dt * i
            store_m([t, RB.mx,RB.my,RB.mz, RB.mx2(),RB.my2(),RB.mz2(), RB.energy_x(),RB.energy_y(),RB.energy_z(),RB.Sin(),RB.Ein(), RB.Phi_x(), RB.Phi_z()])
        if i % (N/10) == 0:
            print (i+0.0)/N*100, "%"
            print "|m| = ", RB.m_magnitude()
            print "E = ", RB.energy()
        
finally:
    mfile.close()

print "Final time = ", N*dt
print "Final |m| = ", RB.m_magnitude()
print "Final kinetic E = ", RB.energy()
print "Final internal entropy  = ", RB.sin
print "Final internal energy = ", RB.Ein()
print "Final total energy = ", RB.Etot()
print "Final E_kin_x = ", RB.energy_x()
print "Final E_kin_y = ", RB.energy_y()
print "Final E_kin_z = ", RB.energy_z() 
print "Final Phi_z = ", RB.Phi_z()


#import matplotlib.pyplot as plt
#from mpl_toolkits.mplot3d import Axes3D

#def print_mm(filter_step = 1):
#    fig = plt.figure()
#    ax = fig.add_subplot(111, projection='3d')
#
#    ax.plot(mx, my, mz, label='m')
#    #ax.plot(filter(mx,filter_step), filter(my,filter_step), filter(mz,filter_step), label='m')
#    ax.set_xlabel("mx")
#    ax.set_ylabel("my")
#    ax.set_zlabel("mz")
#    ax.legend()
#
#    plt.plot(m)
#plt.show()

